local header = { 
    0x46, 0x4F, 0x52, 0x4D, 
    0x00, 0x00, 0x00, 0x00, 
    0x44, 0x50, 0x53, 0x31, 
    0x53, 0x4D, 0x50, 0x41 };

local unknown = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00 };

function wb32(f, x)
    local b4 = string.char(x % 256) x = (x - x % 256) / 256
    local b3 = string.char(x % 256) x = (x - x % 256) / 256
    local b2 = string.char(x % 256) x = (x - x % 256) / 256
    local b1 = string.char(x % 256) x = (x - x % 256) / 256
    f:write(b1, b2, b3, b4)
end

function wb16(f, x)
    local b4 = string.char(x % 256) x = (x - x % 256) / 256
    local b3 = string.char(x % 256) x = (x - x % 256) / 256
    f:write(b3, b4)
end

function wb(f, x)
    f:write(string.char(x))
end

function wb_table(f, data)
    for k, v in ipairs(data) do
        wb(f, v);
    end
end

function w_slices(f, slices)
    for k, slice in ipairs(slices) do
        wb32(f, slice.start_point);
        wb32(f, slice.end_point);
        wb32(f, slice.loop_point);
    end
end

function make_ot_table(sample)
    local sample_buffer = sample.sample_buffer
    local slice_count = table.getn(sample.slice_markers)
    local sample_len = sample_buffer.number_of_frames;

    -- print("\n--------------------------------");
    -- print("sample name: \""..sample.name.."\"");
    print("sample length: "..sample_len);
    -- print("slice_count: "..slice_count);
  
    local ot = {};

    -- Insert header and unknown
    for k,v in ipairs(header) do
        table.insert(ot, v);
    end

    for k,v in ipairs(unknown) do
        table.insert(ot, v);
    end

    -- tempo (32)
    table.insert(ot, 0x0b40);
    -- trim_len (32)
    table.insert(ot, sample_len);
    -- loop_len (32)
    table.insert(ot, sample_len);
    -- stretch (32)
    table.insert(ot, 0x00);
    -- loop (32)
    table.insert(ot, 0x00);
    -- gain (16)
    table.insert(ot, 0x30);
    -- quantize (8)
    table.insert(ot, 0xff);
    -- trim_start (32)
    table.insert(ot, 0x00);
    -- trim_end (32)
    table.insert(ot, sample_len);
    -- loop_point (32)
    table.insert(ot, 0x00);

    -- Checksum only includes the size of everything *after* header and unknown
    --               Size of gain
    --                        Size of quantize
    --                                Size of the other 8 values
    local checksum = (16/8) + (8/8) + ((32/8) * 8);

    for k, v in ipairs(sample.slice_markers) do
        local next = k < slice_count and 
            sample.slice_markers[k + 1] or 
            sample_len
            
        print("v: "..v..", next: "..next);

        local s_start = v - 1;
        local s_end = next - 1;

        -- start_point (32)
        table.insert(ot, s_start);
        -- end_point (32)
        table.insert(ot, s_end);
        -- loop_point (32)
        table.insert(ot, 0xffffffff);

        -- Add the size of the 3 values above
        checksum = checksum + ((32/8) * 3);

        -- print("[ "..k.." ] start: "..s_start..", end: "..s_end.. ", length: "..length);
    end
    
    for i=1,64-slice_count do
         -- start_point (32)
        table.insert(ot, 0x00000000);
        -- end_point (32)
        table.insert(ot, 0x00000000);
        -- loop_point (32)
        table.insert(ot, 0x00000000);

        -- Add the size of the 3 values above
        checksum = checksum + ((32/8) * 3);
    end

    -- slice_count (32)
    table.insert(ot, slice_count);
    -- Add size of slice_count and checksum
    checksum = checksum + (32/8) + (16/8);
    print("checksum: "..checksum);
    -- checksum (16)
    table.insert(ot, checksum);

    return ot;
end

function write_ot_file(filename, ot)
    local name = filename:match("(.+)%..+$")
    local f = io.open(name..".ot", "wb");
    for k,v in ipairs(ot) do
        if k <= 23 or k == 30 then
            wb(f, v);
        elseif k == 29 or k == #ot then
            wb16(f, v);
        else
            wb32(f, v);
        end
    end
    f:close();
end
